/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.connector.jms.activemq.embedded;

import java.util.Properties;

import javax.jms.ConnectionFactory;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;

import org.apache.commons.beanutils.BeanUtils;
import org.bluestemsoftware.specification.eoa.ext.connector.ConnectorException;
import org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnector;
import org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnectorFactory;
import org.bluestemsoftware.specification.eoa.system.container.ConnectionSupport;

public final class JMSConnectorFactoryImpl implements JMSConnectorFactory.Provider {

    private static final long serialVersionUID = 1L;

    public static String NAME = "activemq-embedded-connector";

    public JMSConnectorFactoryImpl() {
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.ExtensionFactory$Provider#spi_getName()
     */
    public String spi_getName() {
        return NAME;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.connector.ConnectorFactory$Provider#spi_getConnectorImpl()
     */
    public String spi_getConnectorImpl() {
        return JMSConnectorImpl.IMPL;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.connector.jms.JMSConnectorFactory$Provider#spi_createConnector(java.lang.String, org.bluestemsoftware.specification.eoa.system.container.ConnectionSupport, java.util.Properties, boolean)
     */
    public JMSConnector.Provider spi_createConnector(String name, ConnectionSupport connectionSupport, Properties properties, boolean isXA) throws ConnectorException {

        ManagedConnectionFactoryImpl mcfi = new ManagedConnectionFactoryImpl(isXA);
        if (properties != null) {
            try {
                BeanUtils.populate(mcfi, properties);
            } catch (Throwable th) {
                throw new ConnectorException("Error setting properties on Connector of type '"
                        + JMSConnector.TYPE
                        + "' and impl '"
                        + JMSConnectorImpl.IMPL
                        + "'. "
                        + th);
            }
        }

        ConnectionFactory cf = null;
        try {
            cf = (ConnectionFactory)mcfi.createConnectionFactory((ConnectionManager)connectionSupport);
        } catch (ResourceException re) {
            throw new ConnectorException("Error creating connector. " + re);
        }

        return new JMSConnectorImpl(cf);
        
    }

}
